import torch
import numpy as np
import numpy.linalg as LA
from EIVGeneral.coverage_metrics import logical_and_along_dimension, multivariate_interval_length


def compute_par_est_var(x_train, y_train, sigma_y, design_matrix, device):
    X = design_matrix(x_train, device=device)
    y = y_train
    assert len(X.shape) == 2 and len(y_train.shape) == 2
    X_T = torch.transpose(X, 0,1)
    inv_X_T_X = torch.linalg.inv(torch.matmul(X_T, X))
    par_est = torch.matmul(inv_X_T_X,
            torch.matmul( X_T, y))
    par_var = inv_X_T_X * sigma_y**2
    return par_est, par_var

def linear_pred_unc(x_train, y_train, sigma_y, design_matrix, x_test, device):
    par_est, par_var = compute_par_est_var(x_train, y_train, sigma_y, design_matrix, device=device)
    X = design_matrix(x_test, device=device)
    X_T = torch.transpose(X, 0,1)
    pred = torch.matmul(X, par_est)
    # assume univariate, for simplicity
    if len(pred.shape) > 1:
        assert len(pred.shape) == 2
        assert pred.shape[1] == 1
    else:
        pred = pred.view((-1,1))
    cov = torch.matmul(X, torch.matmul(par_var, X_T))
    unc = torch.sqrt(torch.diag(cov)).view((-1,1))
    assert pred.shape == unc.shape
    return pred, unc


def linear_coverage(pred, epis_unc, true_y, q=0.95):
    out_dim = true_y.shape[1]
    interval_length = multivariate_interval_length(dim=out_dim, q=q) \
                * epis_unc
    # numerical computation
    errors = pred - true_y
    # assume univariate for simplicity
    assert len(errors.shape) == 2
    assert errors.shape[1] == 1
    check_if_in_interval = logical_and_along_dimension(
            torch.abs(errors) <= interval_length, dim=1)
    numerical_coverage = torch.mean(check_if_in_interval.to(torch.float32)
            ).cpu().detach().item()
    return numerical_coverage
    



