from operator import itemgetter
from torch.utils.data import Dataset


class VerticalCut(Dataset):
    """
    Only include certain elements of the dataset `dataset`, namely those with
    index in `components_to_pick`.
    :param dataset: A torch.utils.data.Dataset
    :param components_to_pick: List, defaults to [0,].
    """
    def __init__(self, dataset, components_to_pick=[0,]):
        super(VerticalCut, self).__init__()
        self.dataset = dataset
        self.components_to_pick = components_to_pick

    def __getitem__(self, i):
        return itemgetter(*self.components_to_pick)(self.dataset.__getitem__(i))

    def __len__(self):
        return len(self.dataset)

    @property
    def tensors(self):
        return itemgetter(*self.components_to_pick)(self.dataset.tensors)
