import os
import glob
import json

metrics_to_display = ['rmse', 'coverage_numerical', 
        'true_coverage_numerical', 'total_coverage']
show_incomplete = True

list_of_result_files = glob.glob(os.path.join('results','*.json'))
results = {}
for filename in list_of_result_files:
    data = filename.replace(os.path.join('results','metrics_'),'').replace('.json','')
    with open(filename,'r') as f:
       results[data] = json.load(f)

def save_readout(dictionary, key):
    """
    Returns the value of the `dictionary` for `key`, unless
    the later doesn't exist, in which case (None,None) is returned.
    """
    try:
        readout = dictionary[key]
        if type(readout) is list:
            assert len(readout) == 2
            return readout
        else:
            readout = float(readout)
            return (readout, None)
    except KeyError:
        return (None,None)

## header
header_string = 'DATA                '
offset = 20
for metric in metrics_to_display:
    header_string += f'     {metric}    '
print(header_string)
print(offset * '_' + 70 * '_')
## results
for data in results.keys():
    noneiv_results = [save_readout(results[data]['noneiv'], metric)
            for metric in metrics_to_display]
    noneiv_row_name  = f'{data} - nonEiV:'
    noneiv_results_string = noneiv_row_name + (offset - len(noneiv_row_name)) * ' '
    for [metric_mean, metric_std] in noneiv_results:
        if metric_mean is None:
            noneiv_results_string += '  None (None)'
        else:
            if metric_std is not None:
                noneiv_results_string += f'  {metric_mean:.3f} ({metric_std:.3f})'
            else:
                noneiv_results_string += f'  {metric_mean:.3f} (NaN)'
    print(noneiv_results_string)
    eiv_results = [save_readout(results[data]['eiv'],metric)
            for metric in metrics_to_display]
    eiv_row_name  = f'{data} - EiV:'
    eiv_results_string = eiv_row_name + (offset - len(eiv_row_name)) * ' '
    for [metric_mean, metric_std] in eiv_results:
        if metric_mean is None:
            eiv_results_string += '  None (None)'
        else:
            if metric_std is not None:
                eiv_results_string += f'  {metric_mean:.3f} ({metric_std:.3f})'
            else:
                eiv_results_string += f'  {metric_mean:.3f} (NaN)'
    print(eiv_results_string)
    print(offset * '_' + 70 * '_')


