"""
Plot predictions with uncertainties for (simulated) datasets with a ground 
truth. At the moment it is assumed that the used datasets have an output
dimension equal to 1. Plots are only produced for datasets with input dimension
and output dimension 1.
"""
import importlib
import os
import json

import torch

import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from EIVArchitectures import Networks
from EIVTrainingRoutines import train_and_store
from plot_prediction import compute_predictions_and_uncertainties, data_list, list_x_range, list_number_of_draws
from plot_coverage_vs_q import colors

font = {'family' : 'DejaVu Sans',
        'weight' : 'normal',
        'size'   : 16*0.80}

matplotlib.rc('font', **font)


list_x_range = [torch.linspace(-1.0,1.0, 200),
                torch.linspace(-1.0,1.0, 200),
                torch.linspace(-1.0,1.0, 200),
        torch.linspace(-0.2,0.8, 200)]
plt.figure(1)
plt.clf()
for data, x_range, number_of_draws, color in zip(data_list,
        list_x_range, list_number_of_draws, colors):
    eiv_plotting_dictionary = compute_predictions_and_uncertainties(
            data=data,
            x_range=x_range,
            eiv=True,
            number_of_draws=number_of_draws[0])
    noneiv_plotting_dictionary = compute_predictions_and_uncertainties(
            data=data,
            x_range=x_range,
            eiv=False,
            number_of_draws=number_of_draws[1])
    _, y_values = eiv_plotting_dictionary['range_points']
    noisy_x_points, _ = eiv_plotting_dictionary['noisy_range_points']
    func = eiv_plotting_dictionary['func'] 
    func_noisy_x_points = func(torch.tensor(noisy_x_points)).numpy()
    y_values, func_noisy_x_points = y_values.flatten(), func_noisy_x_points.flatten()
    eiv_pred = eiv_plotting_dictionary['prediction'].flatten()
    noneiv_pred = noneiv_plotting_dictionary['prediction'].flatten()
    assert eiv_pred.shape == noneiv_pred.shape
    assert y_values.shape == eiv_pred.shape
    assert func_noisy_x_points.shape == eiv_pred.shape
    eiv_true_deviation = np.abs(eiv_pred - y_values).flatten()
    eiv_noisy_deviation = np.abs(eiv_pred - func_noisy_x_points).flatten()
    noneiv_true_deviation = np.abs(noneiv_pred - y_values).flatten()
    noneiv_noisy_deviation = np.abs(noneiv_pred - func_noisy_x_points).flatten()
    plt.loglog(noneiv_true_deviation, noneiv_noisy_deviation,'x', color=color, markersize=5.0)
    plt.loglog(eiv_true_deviation, eiv_noisy_deviation,'+', color=color, markersize=5.0)


plt.ylim(bottom=1e-4)
plt.xlim(left=1e-4)
plt.xlabel(r'$|$'+'prediction'+r'$-g(\zeta)$'+r'$|$') 
plt.ylabel(r'$|$'+'prediction'+r'$-g(x)$'+r'$|$') 

        


# plt.ylim([0,1.0])
# plt.xlim([0,1.0])
x_values = np.linspace(0, 1)
plt.plot(x_values, x_values, linestyle='dotted', color='k')
plt.gca().set_aspect('equal', adjustable='box')
plt.tight_layout()
plt.savefig(f'results/figures/deviation_scatter.pdf')
plt.show()
